/****************************************************************
*Fichier cree par :                                             *
*                 Selma Abdelrahim                              *
*                 Farzana Chutkai                               *
*                 Denis Berthod                                 *
*Pour le projet du cours de systeme du premier semestre         *
*de l'anne 2003/2004 : Le systeme IFS                          *
*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <string.h>

#include "blocio.h"
#include "ifs_error.h"
#include "ifs_util.h"
#include "ifs_tables.h"
#include "ifs_types.h"

//fonction d'affichage d'un bloc sous forme hexadecimale


void affblk(bloc blk){
  int j;
  for(j=0;j<B_SIZE;j++){
      printf("%s",ctoh(blk[j]));
      if(j % 16 == 15){
	printf("\n");
      } else{
	printf(" ");
      }
    }
    printf("\n");
}






//fonction ecrivant un bloc
int writeb(int descripteur,bloc bloc_ecr,USHORT no_bloc){
  int nbEcrit;
  //le disque n'est pas extensible, il faut vrif si l'emplacement pour le bloc existe
  /*if(nbBlocs(descripteur)<= no_bloc){
    ifs_err=BAD_BLOC_ALLOC;
    return -1;
    }*/
  if(lseek(descripteur,no_bloc*128,SEEK_SET)==-1) fatal_error("blocio : writeb : lseek");
  if((nbEcrit=write(descripteur,bloc_ecr,B_SIZE*sizeof(char)))==-1)fatal_error("blocio :writeb: write");
  //affblk(bloc_ecr);//debug
  if(nbEcrit != B_SIZE){
    printf("writeb :le nombre d'octet ecrit ne correspond pas  la longueuer d'un bloc\n");
    ifs_err=PARTIAL_BLOC_WRITE;
    return -1;
  }
  return 1;
}



//fonction lisant un bloc
int readb(int descripteur,bloc buffer,USHORT no_bloc){
  int nbLu;
  //le disque n'est pas extensible, il faut vrif si l'emplacement pour le bloc existe
  /* if(nbBlocs(descripteur)<= no_bloc){
    ifs_err=BAD_BLOC_ALLOC;
    return -1;
    }*/
  if(lseek(descripteur,no_bloc*128,SEEK_SET)==-1) fatal_error("blocio : writeb : lseek");
  if((nbLu=read(descripteur,buffer,B_SIZE*sizeof(char)))==-1)fatal_error("blocio :writeb: write");
  if(nbLu != B_SIZE){
    ifs_err=PARTIAL_BLOC_READ;
    return -1;
  }
  return 1;
}


//fonction ecrivant un bloc dans la table des caches
int cache_writeb(USHORT disc_num,USHORT bloc_num,bloc blc){
  int indice = HASH_CACHE(disc_num,bloc_num);
  cache buf,*prec;
  char *name;
  int descr,i;

  buf.free=0;
  buf.disc_num=disc_num;
  buf.bloc_num=bloc_num;
  memcpy(buf.bloc,blc,B_SIZE);

  prec=&cache_table[indice];
  /*si le bloc dja  cette place est vide ou est le mme bloc, on l'ecrase sans menagement*/
  if ((CACHE_VIDE(prec) || ((disc_num==prec->disc_num)&&(bloc_num==prec->bloc_num)))){
    memcpy(&cache_table[indice],&buf,sizeof(buf));
  }else{/*sinon on recopie d'abord le bloc precedent sur le disque*/
    name=concat((char *)disc_table[prec->disc_num].name,".ifs");
    if((descr=open(name,O_WRONLY))==-1) return -1;
    //printf("write cache : bloc_num = %d bloc_num_ nouveau = %d\n",prec->bloc_num,bloc_num);
    if(writeb(descr,prec->bloc,prec->bloc_num)==-1) return -1;
    if(close(descr)==-1) return -1;
    memcpy(&cache_table[indice],&buf,sizeof(buf));
  }
  return 1;
}

/*fonction lisant un bloc en regardant si il n'exite pas dans la table des buffers 
 *si le bloc n'etait pas dans la table des buffer alors elle le charge dans cette table.
 */
int cache_readb(USHORT disc_num,USHORT bloc_num,bloc blc){
  int indice = HASH_CACHE(disc_num,bloc_num);
  char *name;
  cache buf,*prec;
  int descr;


  prec=&cache_table[indice];

  /*si le bloc dja  cette place est vide ou n'est pas le mme bloc, on recopie le precedent
   *s'il nest pas vide, on charge le nouveau bloc et on le lit
   */
  if ((CACHE_VIDE(prec) || !((disc_num==prec->disc_num)&&(bloc_num==prec->bloc_num)))){
    if(!CACHE_VIDE(prec)){
      name=concat((char *)disc_table[prec->disc_num].name,".ifs");
      if((descr=open(name,O_WRONLY))==-1) return -1;
      //printf("read cache : bloc_num precedent = %d bloc_num_ nouveau = %d\n",prec->bloc_num,bloc_num);
      if(writeb(descr,prec->bloc,prec->bloc_num)==-1) return -1;
      if(close(descr)==-1) return -1;
    }
    buf.free=0;
    buf.disc_num=disc_num;
    buf.bloc_num=bloc_num;
    name=concat((char *)disc_table[disc_num].name,".ifs");
    if((descr=open(name,O_RDONLY))==-1) return -1;
    if(readb(descr,buf.bloc,buf.bloc_num)==-1) return -1;
    if(close(descr)==-1) return -1;
    memcpy(&cache_table[indice],&buf,sizeof(buf));
  }
  /* ici le cache contient le bloc que l'on veut lire*/
  memcpy(blc,cache_table[indice].bloc,B_SIZE);
  return 1;
}


/*
int main(int argc,char *argv[]){
  bloc b1,b2,b3,b4,b5,b6;
  char *bp;
  int descr,i;
  char eof='f';

  if((descr=open("dsktmp",O_RDWR|O_CREAT|O_TRUNC,S_IRWXU))==-1) fatal_error("main : open");
  if(lseek(descr,500,SEEK_SET)==-1) fatal_error("main : lseek");
  if(write(descr,&eof,1)==-1) fatal_error("main : write eof");
  for(i=0;i<B_SIZE;i++){
    b1[i]='a';
  }
  for(i=0;i<B_SIZE;i++){
    b2[i]='b';
  }
  for(i=0;i<B_SIZE;i++){
    b3[i]='c';
  }
  for(i=0;i<B_SIZE;i++){
    b4[i]='d';
  }
  for(i=0;i<B_SIZE;i++){
    b5[i]='e';
  }
  affblk(b2);
  bp=concat(b5,b2);
  if(writeb(descr,b1,0)==-1) ifs_fatal_error("main : writeb");
  if(writeb(descr,b3,1)==-1) ifs_fatal_error("main : writeb");
  if(writenb(descr,bp,2,1)==-1) ifs_fatal_error("main : writeb");
  if(readb(descr,b6,1)==-1)ifs_fatal_error("main:readb");
  affblk(b6);
  if(close(descr)==-1) fatal_error("main : close");
}

*/
