/****************************************************************
*Fichier cree par :                                             *
*                 Selma Abdelrahim                              *
*                 Farzana Chutkai                               *
*                 Denis Berthod                                 *
*Pour le projet du cours de systeme du premier semestre         *
*de l'anne 2003/2004 : Le systeme IFS                          *
*****************************************************************/

#include "ifs_types.h"
#include "ifs_kernelio.h"
#include <stdlib.h>
#include "ifs_api.h"
#include <fcntl.h>



/************************** fonctions ifs sur les repertoires **********************/

/*ouverture*/
desc_t ifs_opendir(char *reference){
  requete query;
  reponse rep;

  query.code=OPENDIR;
  strcpy(query.args,reference);
  send(&query);
  receive(&rep);

  if(rep.err==-1) return -1;
  return rep.msg_int;
}


/*lecture des entres de repertoire*/
entreerep_p ifs_readdir(desc_t descripteur){
  requete query;
  reponse rep;


  query.code=READDIR;
  query.args_int=descripteur;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return NULL;
  dir_entr.numero=rep.msg_num[0];
  strcpy(dir_entr.nom,rep.msg);
  return (entree_repertoire *)&dir_entr;
}

/*fermeture*/
int ifs_closedir(desc_t descripteur){
  requete query;
  reponse rep;

  query.code=CLOSEDIR;
  query.args_int=descripteur;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*creation*/
int ifs_mkdir(char *reference){
  requete query;
  reponse rep;

  query.code=MKDIR;
  strcpy(query.args,reference);
  printf("ifs_mkdir : envoie de la requete\n");
  send(&query);
  receive(&rep);
  printf("ifs_mkdir : reception de la reponse\n");
  if(rep.err==-1) return -1;
  return 1;
}


/*changement du repertoire courant*/
int ifs_chdir(char *ref){
  requete query;
  reponse rep;

  query.code=CHDIR;
  strcpy(query.args,ref);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}


/*effacement*/
int ifs_rmdir(char *reference){
  requete query;
  reponse rep;

  query.code=RMDIR;
  strcpy(query.args,reference);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/****************************************fonctions de gestion des disques**********************/
/*chagement du disque courant*/
int ifs_chdisc(char *disc_name){
  requete query;
  reponse rep;

  query.code=CHDISC;
  strcpy(query.args,disc_name);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*chagement du disque courant en passant son numero*/
int ifs_chdisc_num(int disc_num){
  requete query;
  reponse rep;

  query.code=CHDISCNUM;
  query.args_int=disc_num;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*renvoie le numero du disc courant*/
int curdisc_num(){
  requete query;
  reponse rep;

  query.code=CURDISC;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return rep.msg_int;
}

/**************************************fonction de consultation des inodes***************************/
/*consultation de l'inode par le nom*/
int ifs_stat(char *reference, stat_p buf){
  requete query;
  reponse rep;

  query.code=STAT;
  strcpy(query.args,reference);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  buf->disc_num=rep.msg_num[0];
  buf->node_num=rep.msg_num[1];
  buf->type=rep.msg_num[2];
  buf->liens=rep.msg_num[3];
  buf->icone=rep.msg_num[4];
  buf->taille=rep.msg_num[5];
  return 1;
}

/*recuperation des statistiques d'un noeud par le descripteur*/
int ifs_fstat(desc_t descripteur,stat_p buf){
  requete query;
  reponse rep;

  query.code=FSTAT;
  query.args_int=descripteur;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  buf->disc_num=rep.msg_num[0];
  buf->node_num=rep.msg_num[1];
  buf->type=rep.msg_num[2];
  buf->liens=rep.msg_num[3];
  buf->icone=rep.msg_num[4];
  buf->taille=rep.msg_num[5];
  return 1;
}

/**************************************fonction de gestion des liens vers les icones****************/
/*lier un fichier a une icone*/
int ifs_linkicone(char *reference,char *icone){
  requete query;
  reponse rep;
  
  query.code=LNICO;
  strcpy(query.args,(char *)concat(concat(reference," "),icone));
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*supprimer la liaison entre une icone et un fichier*/
int ifs_unlinkicone(char *reference){
  requete query;
  reponse rep;

  query.code=ULNICO;
  strcpy(query.args,reference);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/***********************************fonction de gestion des fichiers****************************/

/*ouverture d'un fichier*/
desc_t ifs_open(char *reference,int mode,int type){
  requete query;
  reponse rep;

  query.code=FOPEN;
  strcpy(query.args,reference);
  query.args_num[0]=mode;
  query.args_num[1]=type;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return rep.msg_num[0];
}

/*ouverture d'un noeud d'icone*/
desc_t ifs_open_iconode(int node_num,int mode){
  requete query;
  reponse rep;

  query.code=INOPEN;
  query.args_num[0]=node_num;
  query.args_num[1]=mode;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return rep.msg_num[0];
}

/*fermeture d'un fichier*/
int ifs_close(desc_t descripteur){
  requete query;
  reponse rep;

  query.code=FCLOSE;
  query.args_int=descripteur;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*liaison physique d'un fichier*/
int ifs_link(char *src,char *dst){
  requete query;
  reponse rep;

  query.code=LN;
  strcpy(query.args,(char *)concat(concat(src," "),dst));
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}

/*supression d'une liaison phsique d'n fichier et suppression du fichier le cas echeant*/
int ifs_unlink(char *src){
  requete query;
  reponse rep;

  query.code=UNLN;
  strcpy(query.args,src);
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return 1;
}
 

/*ecriture dans un fichier*/
int ifs_write(desc_t descripteur,void *tampon,int taille){
  int descr;
  char *buffer=(char *)tampon;
  requete query;
  reponse rep;
  //printf("write : les 3 premier caractre du tampon envoye sont %c %c %c\n",buffer[0],buffer[1],buffer[2]);
  /*creation du tube*/
  if(mkfifo(IFS_RW,0666)==-1) return -1;
  query.code=WR;
  query.args_num[0]=descripteur;
  query.args_num[1]=taille;
  send(&query);
  /*envoie du tampon a ecrire*/
  //printf("envoi du buffer\n");
  if((descr=open(IFS_RW,O_WRONLY))==-1) return -1;
  //printf("ouverture du tube en ecriture\n");
  if(write(descr,buffer,taille)==-1) return -1;
  //printf("mesage envoye\n");
  if(close(descr)==-1) return -1;

  receive(&rep);
  if(rep.err==-1) return -1;
  return rep.msg_num[0];//retourne le nombre de caracaters ecrits
}

/*lecture dans un fichier*/
int ifs_read(desc_t descripteur,void *tampon,int taille){
  requete query;
  reponse rep;
  int descr;

  if(mkfifo(IFS_RW,0666)==-1)return -1;
  query.code=RD;
  query.args_num[0]=descripteur;
  query.args_num[1]=taille;
  send(&query);
  /*reception du tampon a lire*/
  //printf("reception du buffer\n");
  if((descr=open(IFS_RW,O_RDONLY))==-1) return -1;
  //printf("tube ouvert en lecture\n");
  if(read(descr,tampon,taille)==-1) return -1;
  if(close(descr)==-1) return -1;
  /*destruction du tube*/
  if(unlink(IFS_RW)==-1) return -1;
  receive(&rep);
  if(rep.err==-1) return -1;
  /*fin et retourne le nombre de caractre lus*/
  return rep.msg_num[0];
}

/*changement de la position de lecture ou d'criture dans le fichier*/
int ifs_seek(desc_t descripteur,pos_t position, int mode){
  requete query;
  reponse rep;

  query.code=SEEK;
  query.args_num[0]=descripteur;
  query.args_num[1]=mode;
  query.args_int=position;
  send(&query);
  receive(&rep);
  if(rep.err==-1) return -1;
  return rep.msg_num[0];//returne le nuvel offset
}
