/****************************************************************
*Fichier cree par :                                             *
*                 Selma Abdelrahim                              *
*                 Farzana Chutkai                               *
*                 Denis Berthod                                 *
*Pour le projet du cours de systeme du premier semestre         *
*de l'anne 2003/2004 : Le systeme IFS                          *
*****************************************************************/

#include "ifs_file_util.h"
#include "ifs_error.h"
#include "ifs_disc_util.h"
#include "ifs_tables.h"
#include "ifs_kernelio.h"
#include "ifs_ino_util.h"
#include <string.h>
#include <fcntl.h>
/*fonction qui cree un fichier non existant au prealable et retourne son inode*/
int ifs_creat_file(char *ref,int type){
  char disc_name[32],dir_ref[512],file_name[30];
  int disc_num,node_num,father_node_num,ico;
  inode file_node;
  /*recuperation du nom du disc et de la reference*/
  parse_discref(ref,disc_name,dir_ref);
  /*recuperation du numero du disc*/
  if(*disc_name=='\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_name))==-1) return -1;
  }
  /*recuperation du nom du fichier a creer et du numero du noeud pere*/
  if((father_node_num=newref2ino(disc_num,dir_ref,file_name))==-1) return -1;
  /*recuperation d'un nouvel inode*/
  if((node_num=next_ino(disc_num))==-1)return -1;
  /*creation de la structure d'inode*/
  file_node.etat=NOEUD_UTILISE;
  switch(type){
  case N_ORD:{
    file_node.type=ORDINAIRE;
    /*recuperation du numero de l'icone par defaut*/
    if((ico=ref2ino(disc_num,"/ficico"))==-1) return -1;
    file_node.icone=ico;
    break;
  }
  case N_ICO:file_node.type=ICONE; file_node.icone=0;break;
  default:{
    ifs_err=BAD_CREAT_TYPE;
    return -1;
  }
  }
  file_node.taille=0;
  file_node.liens=0;
  /*ecriture de l'inode*/
  if(write_ino(disc_num,file_node,node_num)==-1) return -1;
  /*creation d'n lien dans le repertoire pere*/
  if(add_dir_entry(disc_num,father_node_num,file_name,node_num)==-1) return -1;
  return node_num;

}

/*fonction qui ouvre un fichier deja creer*/
int ifs_open_file(char *ref,int mode){
  char disc_name[32],file_ref[512];
  int disc_num,file_num,vnode_indice,i;
  inode ino;
  mem_node vnode;
  open_file o_file;
  
  /*recuperation du nom du disque et du chemin*/
  parse_discref(ref,disc_name,file_ref);
  /*recuperation du numero du disque et du numero d'inode*/
  if(*disc_name =='\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_name))==-1) return -1;
  }
  if((file_num=ref2ino(disc_num,file_ref))==-1) return -1;
  /*regarder si l'inode n'est pas en memoire*/
  /*s'il n'est pas dans la memoire on cree une structure mem_node et on l'insere dans la table*/
  if( node_in_mem(disc_num,file_num)== -1){
    if(read_ino(disc_num,&ino,file_num)==-1) return -1;
    vnode.open_cpt=0;
    vnode.disc_num=disc_num;
    vnode.inode_num=file_num;
    vnode.node=ino;
    /*recherche d'un emplacement libre dans la table*/
    for(vnode_indice=0;vnode_indice<MAX_NODE;vnode_indice++){
      if(inode_table[vnode_indice].open_cpt==0){
	inode_table[vnode_indice]=vnode;
	break;
      }
    }
    if(vnode_indice == MAX_NODE){
      ifs_err=TOO_MANY_INODE_IN_MEM;
      return -1;
    }
  }
  /*creation d'une reference dans la table des fichiers ouverts*/
  o_file.free=0;
  o_file.mode=mode;
  o_file.offset=0;
  o_file.mem_node_num=vnode_indice;
  for(i=0;i<MAX_OPEN;i++){
    if(IS_OPEN_FREE(&o_file_table[i])){
      o_file_table[i]=o_file;
      break;
    }
  }
  if(i == MAX_OPEN){
    ifs_err=TOO_MANY_OPEN_FILES;
    return -1;
  } 
  inode_table[o_file_table[i].mem_node_num].open_cpt++;
  /*retourne l'indice dans la table des fichiers ouverts*/
  return i;
}

/*fonction qui ouvre un noeud deja creer, ne marche qu'avec les fichier icne*/
int open_iconode(int node_num,int mode){
  int vnode_indice,i;
  inode ino;
  mem_node vnode;
  open_file o_file;
  /*verification que c'est bien un noeud d'icone*/
  if(read_ino(cur_disc,&ino,node_num)==-1) return -1;
  if(!IS_ICO(&ino)){
    ifs_err=OPEN_NODE_NO_ICO;
    return -1;
  }
  /*regarder si l'inode n'est pas en memoire*/
  /*s'il n'est pas dans la memoire on cree une structure mem_node et on l'insere dans la table*/
  if( (vnode_indice=node_in_mem(cur_disc,node_num))== -1){
    vnode.open_cpt=0;
    vnode.disc_num=cur_disc;
    vnode.inode_num=node_num;
    vnode.node=ino;
    /*recherche d'un emplacement libre dans la table*/
    for(vnode_indice=0;vnode_indice<MAX_NODE;vnode_indice++){
      if(inode_table[vnode_indice].open_cpt==0){
	inode_table[vnode_indice]=vnode;
	break;
      }
    }
    if(vnode_indice == MAX_NODE){
      ifs_err=TOO_MANY_INODE_IN_MEM;
      return -1;
    }
  }
  /*creation d'une reference dans la table des fichiers ouverts*/
  o_file.free=0;
  o_file.mode=mode;
  o_file.offset=0;
  o_file.mem_node_num=vnode_indice;
  for(i=0;i<MAX_OPEN;i++){
    if(IS_OPEN_FREE(&o_file_table[i])){
      o_file_table[i]=o_file;
      break;
    }
  }
  if(i == MAX_OPEN){
    ifs_err=TOO_MANY_OPEN_FILES;
    return -1;
  } 
  inode_table[o_file_table[i].mem_node_num].open_cpt++;
  /*retourne l'indice dans la table des fichiers ouverts*/
  return i;
}



/*fonction booleenne renvoyait l'indice dans la table des noeud si le neoud est deja en memoire -1 si il n'y est pas */
int node_in_mem(int disc_num,int node_num){
  mem_node vnode;
  int vnode_indice;
  for(vnode_indice=0;vnode_indice<MAX_NODE;vnode_indice++){
    vnode=inode_table[vnode_indice];
    if((vnode.inode_num == node_num)&&(vnode.disc_num==disc_num)&&(vnode.open_cpt!=0)){
      return vnode_indice;
    }
  }
  return -1;
}

/*fonction supprimant le contenu d'un fichier et liberant la memeoire associ mais ne supprimantpas le noeud*/
int ifs_del_file_content(char *ref){
  char disc_name[32],file_ref[512];
  int disc_num,file_num,i,mem_ind,in_mem;
  inode ino;
  byte_addr b_addr;
  data_index d_index;
  printf("entre dans ifs_del_file_content\n");
  /*recuperation du nom du disc et du nom du fichier*/
  parse_discref(ref,disc_name,file_ref);
  /*recuperation du numero du disque et du numero d'inode*/
  if(*disc_name =='\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_name))==-1) return -1;
  }
  if((file_num=ref2ino(disc_num,file_ref))==-1) return -1;

 
  /*recuperation du noeud soit en memoire soi sur le disque*/
  if((mem_ind=node_in_mem(disc_num,file_num))==-1){ 
    if(read_ino(disc_num,&ino,file_num)==-1) return -1;
    in_mem=0;
  }else{
    ino=inode_table[mem_ind].node;
    in_mem=1;
  }
 /*si la tailles t nulle il n'y a rien a faire*/  
  if(ino.taille == 0) return 1; 
  /*verification que ce n'est pas un repertoire*/
  if(IS_REP(&ino)){
    ifs_err=DEL_DIR_CONTENT;
    return -1;
  }

  /*liberation des bloc utiliss et diminution de la taille*/
  while(ino.taille > 0){
    if(offset2addr(disc_num,file_num,ino.taille-1,&b_addr)==-1) return -1;
    
    /*si le bloc d'index est l'inode*/
    if(b_addr.index==-1){
      for(i=b_addr.indice;i>=0;i--){
	if(new_free_bloc(disc_num,ino.blocs[i])==-1) return -1;
	ino.taille--;
      }
    }else{//le bloc est dans un index
      /*recuperation du bloc d'index*/
      if(read_data_index(disc_num,&d_index,b_addr.index)==-1) return -1;
      for(i=b_addr.indice;i>=0;i--){
	if(new_free_bloc(disc_num,d_index.blocs[i])==-1) return -1;
	ino.taille--;
      }
      /*liberation du bloc index*/
      if(new_free_bloc(disc_num,b_addr.index)==-1) return -1;
    }
  }
  /*ecriture du noeud*/
  if(in_mem){
    inode_table[mem_ind].node.taille=ino.taille;
  }else{
    if(write_ino(disc_num,ino,file_num)==-1) return -1;
  }
  return 1;
}

/*fonction supprimant le contenu d'un fichier et liberant la memeoire associ mais ne supprimantpas le noeud*/
int ifs_del_file_content_num(int disc_num,int file_num){
  
  int i,mem_ind,in_mem;
  inode ino;
  byte_addr b_addr;
  data_index d_index;

  /*recuperation du noeud soit en memoire soi sur le disque*/
  if((mem_ind=node_in_mem(disc_num,file_num))==-1){ 
    if(read_ino(disc_num,&ino,file_num)==-1) return -1;
    in_mem=0;
  }else{
    ino=inode_table[mem_ind].node;
    in_mem=1;
  }
 /*si la tailles t nulle il n'y a rien a faire*/  
  if(ino.taille == 0) return 1; 

  /*verification que ce n'est pas un repertoire*/
  if(read_ino(disc_num,&ino,file_num)==-1) return -1;
  if(IS_REP(&ino)){
    ifs_err=DEL_DIR_CONTENT;
    return -1;
  }

  /*liberation des bloc utiliss et diminution de la taille*/
  while(ino.taille > 0){
    if(offset2addr(disc_num,file_num,ino.taille-1,&b_addr)==-1) return -1;
    
    /*si le bloc d'index est l'inode*/
    if(b_addr.index==-1){
      for(i=b_addr.indice;i>=0;i--){
	if(new_free_bloc(disc_num,ino.blocs[i])==-1) return -1;
	ino.taille--;
      }
    }else{//le bloc est dans un index
      /*recuperation du bloc d'index*/
      if(read_data_index(disc_num,&d_index,b_addr.index)==-1) return -1;
      for(i=b_addr.indice;i>=0;i--){
	if(new_free_bloc(disc_num,d_index.blocs[i])==-1) return -1;
	ino.taille--;
      }
      /*liberation du bloc index*/
      if(new_free_bloc(disc_num,b_addr.index)==-1) return -1;
    }
  }
 /*ecriture du noeud*/
  if(in_mem){
    inode_table[mem_ind].node.taille=ino.taille;
  }else{
    if(write_ino(disc_num,ino,file_num)==-1) return -1;
  }
  return 1;
}




/*fonction qui ferme un fichier*/
int ifs_close_file(desc_t descripteur){
  inode ino;
  mem_node *m_ino;
  m_ino= &inode_table[o_file_table[descripteur].mem_node_num];
  /*decrementation du nombre de fichier ouvert sur ce noeud*/
  m_ino->open_cpt--;
  /*supression de l'entre dans la table des fichier ouverts*/
  o_file_table[descripteur].free=1;
  /*si le nombre de liens est null alors, on suprime le fichier*/
  if(m_ino->node.liens==0){
    if(ifs_del_file_content_num(m_ino->disc_num,m_ino->inode_num)==-1)return-1;
    ino.etat=NOEUD_LIBRE; 
    if(write_ino(m_ino->disc_num,ino,m_ino->inode_num)==-1) return -1;
    if(new_free_node(m_ino->disc_num,m_ino->inode_num)==-1)return -1;
  }else{
    /*ecriture de l'inode dans la table des caches*/
    if(write_ino(m_ino->disc_num,m_ino->node,m_ino->inode_num)==-1) return -1;
  }
  return 1;
}


/*fonction qui ecrit dans un fichier*/
int write_file(desc_t descripteur,void *tampon,int taille){
  int *offset,disc_num,bloc_pos,node_num,nb_ecrit;
  inode *ino;
  char *cur_car;
  byte_addr b_addr;
  bloc blc;
  data_index data_iblc;
  USHORT new_addr,new_index_addr;

  ino = &(inode_table[o_file_table[descripteur].mem_node_num].node);
  offset=&(o_file_table[descripteur].offset);
  disc_num=inode_table[o_file_table[descripteur].mem_node_num].disc_num;
  node_num=inode_table[o_file_table[descripteur].mem_node_num].inode_num;
  /*****verification du mode d'ouverture******/
  if(!IS_ECR(o_file_table[descripteur].mode)){
    ifs_err=BAD_DESCRIPTOR;
    return -1;
  }

  /***** allocation du  premier bloc dans le cas ou a taille est nulle*/
  if(ino->taille == 0){
    if((new_addr=next_bloc(disc_num))==IFS_NULL) return -1;
    ino->blocs[0]=new_addr;
  }


  /*****tant que la taille du fichier est strictement inferieur a l'offset, on remplit le vide avec des zro jusqu'a ce que 
  ******la taille et l'offset soint gaux
  *******/
  //printf( "dans write l'offset est %d\n",*offset);
  while(*offset > ino->taille){
    /*recuperation du numero du bloc contenant l'octet  crire(celuid'indice taille)*/
    if(offset2addr(disc_num,node_num,ino->taille,&b_addr)==-1) return -1;
    /*recuperation du bloc*/
    if(read_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
    /*calcul de l'indice de l'octet dans ce bloc*/
    bloc_pos=ino->taille%128;
    /*tant que le bloc n'est pas plein ou que la taille n'est pas egale  l'offset, on met des 0 dans le bloc*/
    while((bloc_pos < 128) && (*offset > ino->taille)){
      blc[bloc_pos]='\0';
      bloc_pos++;
      ino->taille++;
    }
    /*ecriture du bloc*/
    if(write_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;

    /****allocation du prochain bloc selon les cas si le dernier bloc est plein*/
    if(ino->taille %128 ==0){
      /*il faut creer un bloc d'index a partir de l'inode*/
      if(ino->taille==1280){
	/*creation d'un bloc d'index*/
	if((new_index_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	ino->blocs[10]=new_index_addr;
	/*allocatin d'un nouveau bloc*/
	if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	data_iblc.blocs[0]=new_addr;
	data_iblc.next=IFS_NULL;
	if(write_data_index(disc_num,&data_iblc,new_index_addr)==-1)return -1;
      }else if((ino->taille)%8064 == 1280){//creation d'un bloc d'index a partir d'un index
	/*creation d'un bloc d'index*/
	if((new_index_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	if(read_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	data_iblc.next=new_index_addr;
	if(write_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	/*allocation d'un nouveau bloc*/
	if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	data_iblc.blocs[0]=new_addr;
	data_iblc.next=IFS_NULL;
	if(write_data_index(disc_num,&data_iblc,new_index_addr)==-1)return -1;
      }else{//inutile de creer un nouvel index mais il faut distingu entre etre dans un inode ou un index
	if(b_addr.index == -1){//on est dans un inode
	  /*allocation d'un nouveau bloc*/
	  if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	  ino->blocs[b_addr.indice+1]=new_addr;
	}else{//on est dans un index
	  /*allocation d'un nouveau bloc*/
	  if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	  if(read_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	  data_iblc.blocs[b_addr.indice+1]=new_addr;
	  if(write_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	}
      }
    }
  }
  cur_car=tampon;
  nb_ecrit=0;
  /******tant que l'offcet est strictement infrieur  la taille et que le nombre ecrit ne depasse pas la limite, 
   *on modifie les blocs du fichier sans changer la taille*/
  while((*offset < ino->taille)&&(nb_ecrit < taille)){
    /*recuperation du numero  bloc contena tl'octet a ecrire(celui d'indie offset)*/
    if(offset2addr(disc_num,node_num,*offset,&b_addr)==-1) return -1;
    /*recuperation du bloc*/
    if(read_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
    /*calcul de l'indice de l'octet dans ce bloc*/
    bloc_pos=ino->taille%128;
    /*tant que le bloc n'est pas plein ou que l'offset et la taille soit gale ou qu ele nombre ecrit n'a
     *pas atteint la limite, on change les octets*/
    while((bloc_pos<128)&&(*offset<ino->taille)&&(nb_ecrit < taille)){
      //printf("write : le caracter excrit est %c\n",*cur_car);
      blc[bloc_pos]=*cur_car;
      bloc_pos++;
      (*offset)++;
      cur_car++;
      nb_ecrit++;
    }
    /*ecriture du bloc*/
    if(write_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
  }
  /*ici offset == ino->taille si il y avait ass d'octets a lire*/
  /******quand l'offset est gale a la taille il faut penser  alouer de nouveau bloc qu'en c'est necssaire*/
  while(nb_ecrit < taille){
    /*recuperation du numero  bloc contenant l'octet a ecrire(celui d'indie offset ou taille)*/
    if(offset2addr(disc_num,node_num,*offset,&b_addr)==-1) return -1;
    /*recuperation du bloc*/
    if(read_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
    /*calcul de l'indice de l'octet dans ce bloc*/
    bloc_pos=ino->taille%128;
    /*tant que le bloc n'est pas plein ou que le b a ecrire n'a pas atteint la limi    printf("write : le caracter excrit est %c\n",*cur_car);te, on change les octets*/
    while((bloc_pos<128)&&(nb_ecrit < taille)){
      //printf("write : le caracter excrit est %c\n",*cur_car);
      blc[bloc_pos]=*cur_car;
      bloc_pos++;
      (*offset)++;
      ino->taille++;
      cur_car++;
      nb_ecrit++;
    }
    /*ecriture du bloc*/
    if(write_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
    /*si le dernier bloc est plein, il faut en allou un autre 
     *(copier-coller de la partie effectuant la mme pration prcdemment
     */
    /****allocation du prochain bloc selon les cas si le dernier bloc est plein*/
    if(ino->taille %128 ==0){
      /*il faut creer un bloc d'index a partir de l'inode*/
      if(ino->taille==1280){
	/*creation d'un bloc d'index*/
	if((new_index_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	ino->blocs[10]=new_index_addr;
	/*allocatin d'un nouveau bloc*/
	if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	data_iblc.blocs[0]=new_addr;
	data_iblc.next=IFS_NULL;
	if(write_data_index(disc_num,&data_iblc,new_index_addr)==-1)return -1;
      }else if(ino->taille%8064 == 1280){//creation d'un bloc d'index a partir d'un index
	/*creation d'un bloc d'index*/
	if((new_index_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	if(read_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	data_iblc.next=new_index_addr;
	if(write_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	/*allocation d'un nouveau bloc*/
	if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	data_iblc.blocs[0]=new_addr;
	data_iblc.next=IFS_NULL;
	if(write_data_index(disc_num,&data_iblc,new_index_addr)==-1)return -1;
      }else{//inutile de creer un nouvel index mais il faut distingu entre etre dans un inode ou un index
	if(b_addr.index == -1){//on est dans un inode
	  /*allocation d'un nouveau bloc*/
	  if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	  ino->blocs[b_addr.indice+1]=new_addr;
	}else{//on est dans un index
	  /*allocation d'un nouveau bloc*/
	  if((new_addr=next_bloc(disc_num))==IFS_NULL)return -1;
	  if(read_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	  data_iblc.blocs[b_addr.indice+1]=new_addr;
	  if(write_data_index(disc_num,&data_iblc,b_addr.index)==-1) return -1;
	}
      }
    }
  }
  /*retourne le nombre de caractres ecrits*/
  return nb_ecrit;
}

/*fonction qui lit dans un fichier*/
int read_file(desc_t descripteur,void *tampon,int taille){
  int nb_lu,*offset,disc_num,node_num,bloc_pos;
  char *cur_car;
  inode *ino;
  byte_addr b_addr;
  bloc blc;

  //printf("entre dans readfile\n");
  offset=&(o_file_table[descripteur].offset);
  ino = &(inode_table[o_file_table[descripteur].mem_node_num].node);
  disc_num=inode_table[o_file_table[descripteur].mem_node_num].disc_num;
  node_num=inode_table[o_file_table[descripteur].mem_node_num].inode_num;
  cur_car=tampon;
  nb_lu=0;

  /********verification du descripteur*****/
  if(!IS_LEC(o_file_table[descripteur].mode)){
    ifs_err=BAD_DESCRIPTOR;
    return -1;
  }
  /*tant que le nombre d'octet lu est infrieur  la limie et que l'offset est 
   *infrieur  la taille, on rempli le tampon*/
  while((nb_lu <taille) && (*offset < ino->taille)){
    /*recuperation du numero du bloc contenant l'octet  lire(celuid'indice offset)*/
    if(offset2addr(disc_num,node_num,*offset,&b_addr)==-1) return -1;
    /*recuperation du bloc*/
    //printf("lecture du bloc ---------------------------------------------------");
    if(read_bloc(disc_num,blc,b_addr.bloc_num)==-1) return -1;
    /*calcul de l'indice de l'octet dans ce bloc*/
    bloc_pos=(*offset)%128;
    /*tant que le bloc n'a pas ete lu en entier ou que l'offset n'est pas arriv a la fin du fichier 
     *ou que le nombre limite n'a pas t attient, on lit les octets du bloc*/
    while((bloc_pos<128)&&(*offset<ino->taille)&&(nb_lu < taille)){
      *cur_car=blc[bloc_pos];
      //printf("le caractere lu est %c,l'offset est %d, la taille est %d, la position dans le bloc est %d\n",*cur_car,*offset, ino->taille,bloc_pos);
      bloc_pos++;
      (*offset)++;
      cur_car++;
      nb_lu++;
      //printf("le caractere lu est %c,l'offset est %d, la taille est %d,le nom bre a ecrire est %d,le nombre lu est %d\n",*(cur_car-1),*offset, ino->taille,taille,nb_lu);
    }
  }
  //printf("sort de read file\n");
  return nb_lu;
}

/*fonction qui change l'offset du fichier*/
int seek_file(desc_t descripteur,pos_t position,int mode){
  int pos,*offset,taille;

  offset=&(o_file_table[descripteur].offset);
  taille=inode_table[o_file_table[descripteur].mem_node_num].node.taille;
  /*calcul de la nouvelle position par rapport au debut du fichier*/
  switch(mode){
  case DEBUT: pos=position;break;
  case COURANT:pos=*offset+position;break;
  case FIN:pos=taille+position; break;
  default:ifs_err=BAD_SEEK_MODE;return -1;
  }
  pos=(pos<0)?0:pos;//si pos est ngatif alors on le ramne a 0
  /*remplacement de la valeur dans la table des fichiers ouverts*/
  *offset=pos;
  /*fin : on retourne l'offset courant*/
  return pos;
}
/************************************fonctions serveur****************************************/

/*ouverture d'un fichier*/
void ofile(requete *query,reponse *rep){
  char disc_name[32],file_name[512];
  int disc_num,file_num,desc;
  inode ino;

  /*frecuperation du nom du disque et du nom de la reference*/
  parse_discref(query->args,disc_name,file_name);
  /*on verifie si le fichier exist et on le cree le cas echeant*/
  if(*disc_name=='\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_name))==-1){rep->err=-1;return;}
  }
  if((file_num=ref2ino(disc_num,file_name))==-1){
    /*rattrapage de l'erreur dans le cas du mode creation*/
    if((ifs_err != FILE_DO_NOT_EXISTS)&&(!IS_CRE(query->args[0]))){
      rep->err=-1;
      return;
    }
    ifs_err=0;
    /*creation du fichier*/
    if((file_num=ifs_creat_file(file_name,query->args_num[1]))==-1){rep->err=-1;return;}
  }
  /*verificaion que ce n'est pas un repertoire*/
  if(read_ino(disc_num,&ino,file_num)==-1){rep->err=-1;return;}
  if(IS_REP(&ino)){
    ifs_err=DIR_OPEN;
    rep->err=-1;
    return;
  }
  // printf("le fichier n'est pas un repertoire\n");
  /*si le mode contien M_ZER, on efface le contenu du fichier*/
  if(IS_ZER(query->args_num[0])){
    if(ifs_del_file_content(query->args)==-1){rep->err=-1;return;}
  }
  //printf("le contenu du fichier a ete efface\n");
  /*ouverture du fichier*/
  if((desc=ifs_open_file(query->args,query->args_num[0]))==-1){rep->err=-1;return;}
  rep->msg_num[0]=desc;
  rep->err=0;
}

/*fermeture d'un fichier*/
void cfile(requete *query,reponse *rep){
 if(ifs_close_file(query->args_int)==-1){
    rep->err=-1;
  }else{
    rep->err=0;
  }
}

/*lien physique sur un fichier*/
void lnfile(requete *query,reponse *rep){
  char *cur_dst_name,src_name[512],dst_name[512],disc_src[32],disc_dst[32],ref_src[512],ref_dst[512],dst_newref[40];
  int cpt,disc_num,src_num,dst_dir_num;
  inode ino;
  //printf("entre dans ln file\n"); 
  /*recuperation des references de la sourece et de la destination*/
  cur_dst_name=(char *)query->args;
  cpt=0;
  while((*cur_dst_name != ' ')&&(*cur_dst_name != '\0')){
    cur_dst_name++;
    cpt++;
  }
  if(*cur_dst_name !='\0') cur_dst_name++;
  strncpy(src_name,query->args,cpt);
  src_name[cpt]='\0';
  strcpy(dst_name,cur_dst_name);
  /*recuperation du nom du disc et du chemin pour la source*/
  parse_discref(src_name,disc_src,ref_src);
  /*recuperation du nom du disque et du chemin pour la destination*/
  parse_discref(dst_name,disc_dst,ref_dst);
  /*comparaison des noms des disques : si ils sont diffrent erreur : pas de liens d'un disque a l'autre*/
  if(strcmp(disc_src,disc_dst)){
    ifs_err = DISCS_LN;
    rep->err=-1;
    return;
  }
  //printf("les disques sont les memes le nom est \n");
  /*recuperation du numero dudisque et du numro d'inode pour la source*/
  if(*disc_src == '\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_src)==-1)){rep->err=-1;return;}
  }
  if((src_num=ref2ino(disc_num,ref_src))==-1){rep->err=-1;return;}
  //printf ("pour la source disc_num=%d et node_num=%d\n",disc_num,src_num);
  /*verification que la source n'est pas un repertoire*/
  if(read_ino(disc_num,&ino,src_num)==-1){rep->err=-1;return;}
  if(IS_REP(&ino)){
    ifs_err=DIR_LN;
    rep->err=-1;
    return;
  }
  /*recuperation  du numero d'inode du repertoire pere et du nom de la destination
  *en verifiant que l'entree n'existe pas deja
  */
  if((dst_dir_num=newref2ino(disc_num,ref_dst,dst_newref))==-1){rep->err=-1;return;}
  //printf("le numero de l'inode du reerptoire de destination est %d\n",dst_dir_num);
  /*ajout de l'entre de repertoire*/
  if(add_dir_entry(disc_num,dst_dir_num,dst_newref,src_num)==-1){rep->err=-1;return;}
  /*fin : pas d'erreurs*/
  rep->err=0;

}


/*suppression d'un lien physique et du fichier le cas echeant*/
void unlnfile(requete *query, reponse *rep){
  char disc_name[32],file_ref[512],rep_name[512],file_name[40];
  int disc_num, file_num, dir_num;
  inode ino;

  //printf("entre dans unln\n");
  /*recuperation du nom du disc et de la reference du fichier*/
  parse_discref(query->args,disc_name,file_ref);
  /*recuperation du numero du disc et de l'node du fichier*/
  if(*disc_name=='\0'){
    disc_num=cur_disc;
  }else{
    if((disc_num=discname2num(disc_name))==-1) {rep->err=-1;return;}
  }
  if((file_num=ref2ino(disc_num,file_ref))==-1) {rep->err=-1;return;}
  //printf("le numero du fichier est %d\n",file_num);
  /*recuperation du numero du noeud du repertoire*/
  cutref(file_ref,file_name,rep_name);
  if(*rep_name=='\0') strcpy(rep_name,".");
  //printf("le nom du fichier est %s et le nomdu repertoire est %s\n",file_name ,rep_name);
  if((dir_num=ref2ino(disc_num,rep_name))==-1) {rep->err=-1;return;}
  //printf("le numero du repertoire est %d\n",dir_num);
  /*supression de l'entree de repertoire*/
  if(del_dir_entry(disc_num,dir_num,file_name)==-1) {rep->err=-1;return;}
  /*si le compteur de liens est nul alors on suprime le contenu puis le noeud siil n'est pas en memoire*/
  if(node_in_mem(disc_num,file_num)!=-1){
    if(ifs_del_file_content(query->args)==-1) {rep->err=-1;return;}
    //printf("contenu du fichier supprimer");
    ino.etat=NOEUD_LIBRE; 
    if(write_ino(disc_num,ino,file_num)==-1) {rep->err=-1;return;}
    if(new_free_node(disc_num,file_num)==-1){rep->err=-1;return;}
  }
  /*fin*/
  //printf("fin de unln avec succes\n");
  rep->err=0;
}


/*ecriture dans un fichier*/
void wfile(requete *query,reponse *rep){
  char *buffer;
  int descr,nb_ecrit;

  buffer=(char *)malloc(query->args_num[1]);
  /*lecture du buffer envoye*/
  //printf("reception du bufer\n");
  if((descr=open(IFS_RW,O_RDONLY))==-1) {rep->err=-1;return;}
  //printf("tube ouvert en lecture\n");
  if(read(descr,buffer,query->args_num[1])==-1) {rep->err=-1;return;}
  if(close(descr)==-1) {rep->err=-1;return;}
  /*destriction du tube*/
  if(unlink(IFS_RW)==-1){rep->err=-1;return;}
  //printf("buffer recu\n");
  //printf("les trois preier caracter utampu recu sont %c %c %c\n",buffer[0],buffer[1],buffer[2]);
  /*ecriture du buffer*/
  if((nb_ecrit=write_file(query->args_num[0],buffer,query->args_num[1]))==-1){rep->err=-1;return;}
  /*fin pas d'erreurs,retourner le nombre de caracters ecrits*/
  rep->err=0;
  rep->msg_num[0]=nb_ecrit;
}

/*lecture dans un fichier*/
void rfile(requete *query,reponse *rep){
  void *buffer;
  int descr,nb_lu;

  buffer=(void *)malloc(query->args_num[1]);
 
  /*lecture du buffer*/
  if((nb_lu=read_file(query->args_num[0],buffer,query->args_num[1]))==-1){rep->err=-1;return;}
  /* ecriture du buffer a envoyer*/
  //printf("envoie du buffer\n");
  if((descr=open(IFS_RW,O_WRONLY))==-1) {rep->err=-1;return;}
  //printf("tube ouvert en ecriture\n");
  if(write(descr,buffer,query->args_num[1])==-1) {rep->err=-1;return;}
  if(close(descr)==-1) {rep->err=-1;return;}
  
 
  /*fin pas d'erreurs,retourner le nombre de caracters ecrits*/
  rep->err=0;
  rep->msg_num[0]=nb_lu;

}

/*changement de l'offset*/
void sfile(requete *query, reponse *rep){
  int off;

  if((off=seek_file(query->args_num[0],query->args_int,query->args_num[1]))==-1){rep->err=-1;return;}
  rep->err=0;
  rep->msg_num[0]=off;
}

/*ouverture d'un noeud d'icone*/
void inopen(requete *query, reponse *rep){
  int descr;
  if((descr=open_iconode(query->args_num[0],query->args_num[1]))==-1){rep->err=-1;return;}
  rep->err=0;
 rep->msg_num[0]=descr;
}
