/****************************************************************
*Fichier cree par :                                             *
*                 Selma Abdelrahim                              *
*                 Farzana Chutkai                               *
*                 Denis Berthod                                 *
*Pour le projet du cours de systeme du premier semestre         *
*de l'anne 2003/2004 : Le systeme IFS                          *
*****************************************************************/

#include "lendian.h"
#include "ifs_types.h"
#include "ifs_ino_util.h"
#include "ifs_tables.h"
#include "ifs_error.h"
#include "ifs_kernelio.h"
#include "ifs_disc_util.h"

/*convertin d'un addresse vers un USHORT*/
USHORT to_ino_num(ino_addr *adresse){
  disc *disque=&disc_table[adresse->disc_num];
  USHORT nb_bloc=adresse->bloc_num-disque->inode_addr;
  return  nb_bloc*4+adresse->indice;
}

/*convertion d'un numero d'inode vers une adresse*/
void to_ino_addr(ino_addr *adresse,disc *disque, USHORT ino_num){
  adresse->disc_num=disque->dev_num;
  adresse->bloc_num=disque->inode_addr+ino_num/4;
  adresse->indice=ino_num%4;
}



typedef struct byte_info{
  USHORT nb_free;//nombre d'inode libre dans l'octet(nombre de 1)
  USHORT addr[8];//adresses des inodes libre rfrenc par l'octet
}byte_info;

/* renvoie les addresses des inodes rfrenc par un octets dans un estructure byte_info*/
void read_byte_addr(USHORT bloc_num,USHORT byte_num,unsigned char byte,byte_info *b_info){
  unsigned char mask=128;
  int cpt=0;
  /*initialisation de lastructure*/
  b_info->nb_free=0;
  /*recherche des inodes libre*/
  while(cpt<8){
    while((cpt<8)&&!(mask & byte)){
      mask=mask>>1;
      cpt++;
    }
    b_info->addr[b_info->nb_free]=(bloc_num-2)*1024+byte_num*8+cpt;
    b_info->nb_free++;
    cpt++;
    mask=mask>>1;
  }
  /*debug*/
  //for(cpt=0;cpt<8;cpt++) printf("l'addresse %d de l'octet %d est %d\n",cpt,byte_num,b_info->addr[cpt]);
}

/*initialise les champs quick_inode quick_inode_cpt et quick_inode_ind*/
int quick_acces_init(int read_d,disc *disque){
  bloc ino_map;
  int ino_map_numero,ino_map_nombre,free_cpt,byte_cpt;
  byte_info b_info;
  
  ino_map_nombre=disque->description_size-2;
  ino_map_numero=0;
  free_cpt=0;
  disque->quick_inode_cpt=0;

  if(disque->free_inode_space==0){
    ifs_err=TOO_MANY_INODE;
    return -1;
  }
  
  while((free_cpt<32)&&(ino_map_numero < ino_map_nombre)){
    if(readb(read_d,ino_map,2+ino_map_numero)==-1) return -1;
    byte_cpt=0;
    while((byte_cpt<128)&&(free_cpt<32)){
      if(ino_map[byte_cpt]==0){
	byte_cpt++;
      }else{
	read_byte_addr(2+ino_map_numero,byte_cpt,ino_map[byte_cpt],&b_info);
	byte_cpt ++;
	while((free_cpt<32) && !(b_info.nb_free == 0)){
	  b_info.nb_free--;
	  disque->quick_inode[free_cpt]=b_info.addr[7-b_info.nb_free];
	  disque->quick_inode_cpt++;
	  //printf("l'adresse ecrite est %d \n",disque->quick_inode[free_cpt]);
	  free_cpt ++;
	}
      }
      
    }
    ino_map_numero++;
  }
  disque->quick_inode_ind=disque->quick_inode_cpt-1;
  return 1;
}

int quick_access_refill(USHORT disc_num){
  bloc ino_map;
  int ino_map_numero,ino_map_nombre,free_cpt,byte_cpt;
  byte_info b_info;
  disc *disque;
  disque=disc_table+disc_num;
  ino_map_nombre=disque->description_size-2;
  ino_map_numero=0;
  free_cpt=0;

 if(disque->free_inode_space==0){
    ifs_err=TOO_MANY_INODE;
    return -1;
  }

 disque->quick_inode_cpt=0;

  while((free_cpt<32)&&(ino_map_numero < ino_map_nombre)){
    if(cache_readb(disc_num,2+ino_map_numero,ino_map)==-1) return -1;
    byte_cpt=0;
    while((byte_cpt<128)&&(free_cpt<32)){
      if(ino_map[byte_cpt]==0){
	byte_cpt++;
      }else{
	read_byte_addr(2+ino_map_numero,byte_cpt,ino_map[byte_cpt],&b_info);
	byte_cpt ++;
	while((free_cpt<32) && !(b_info.nb_free == 0)){
	  b_info.nb_free--;
	  disque->quick_inode[free_cpt]=b_info.addr[b_info.nb_free];
	  disque->quick_inode_cpt++;
	  //printf("l'adresse ecrite est %d \n",disque->quick_inode[free_cpt]);
	  free_cpt ++;
	}
      }
 
    }
    ino_map_numero++;
  }
  disque->quick_inode_ind=disque->quick_inode_cpt-1;

  return 1;

}

/*fnction prenant un char et met e bit dsign par son indice en partant de la gauche et ne commencant  compt  0*/
unsigned char bitset(unsigned char byte,int indice,int bit){
  unsigned char mask=(unsigned char)(128 >> (indice));//positionnement du bit

  if(bit==0){
    mask=~mask;
    return byte & mask;
  }else{
    return byte | mask;
  }
}

/*fonction retournant le prochain inode vide et le mettant  occup dans la inode_map et IFS_NULL en cas d'erreur et diminuant le nombre d'inodes libres*/
int  next_ino(USHORT disc_num){
  disc *disque;
  bloc ino_map;
  USHORT resultat;
  USHORT no_bloc,no_byte,no_bit;

  disque=&disc_table[disc_num];
  if(disque->quick_inode_cpt == 0){//si il n'y a plus d'inode en aces  rapide, on rempli la table 
    //printf("reinitialistion de la table des inode en acces rapide\n");
    if(quick_access_refill(disc_num)==-1) return -1;
    //printf("table reinitialise");
  }

  //positionnement du bit 0  la bonne place
  resultat=disque->quick_inode[disque->quick_inode_ind];
  no_bloc=2+resultat/1024;
  no_byte=(resultat%1024)/8;
  no_bit=resultat%8;//le premier bit est numerote par 0
  if(cache_readb(disc_num,no_bloc,ino_map)==-1) return -1;
  //printf("affichagede la table de bit avant changement\n");
  //affblk(ino_map);
  ino_map[no_byte]=bitset(ino_map[no_byte],no_bit,0);
  //printf("affichage de la table de bit aprs changement\n");
  //affblk(ino_map);
  if(cache_writeb(disc_num,no_bloc,ino_map)==-1)return -1;
  //printf("inomap_ecrite sur le disque\n");

  //decrementation de la variable qui compte le nombre d'inode libre
  disque->free_inode_space --;
  //decrementation de l'indice des ino en acces rqpide
  disque->quick_inode_cpt--;
  disque->quick_inode_ind --;
  return resultat;

}

/*fonction rendant un inode allouable*/

int new_free_node(USHORT disc_num,USHORT node_num){
  disc *disque;
  bloc ino_map;
  USHORT no_bloc,no_byte,no_bit;

  /*verification que le numero est valide*/


  disque=&disc_table[disc_num];
  //si la table des innodes en acces rapide n'est pas pleine, on mais le noeud dedans
  if(disque->quick_inode_cpt < 32){
    disque->quick_inode[disque->quick_inode_cpt]=node_num;
    disque->quick_inode_cpt++;
    disque->quick_inode_ind++;
  }

  //positionnement du bit 1  la bonne place
  no_bloc=2+node_num/1024;
  no_byte=(node_num%1024)/8;
  no_bit=node_num%8;//le premier bit est numerote par 0
  if(read_map_bloc(disc_num,ino_map,no_bloc)==-1) return -1;
  //printf("ino_map avant positionement du bit 1\n");
  //affblk(ino_map);
  ino_map[no_byte]=bitset(ino_map[no_byte],no_bit,1);
  //printf("ino_map apres positionement du bit 1\n");
  //affblk(ino_map);
  if(write_map_bloc(disc_num,ino_map,no_bloc)==-1)return -1;

  //incrementation de la variable qui compte le nombre d'inode libre
  disque->free_inode_space ++;
  return 1;
}





int calc_ino_map(bloc ino_map,int nb_ino_restant){
  int i,restant,byte_cpt;
  restant=nb_ino_restant;
  byte_cpt=0;
  while((byte_cpt<128)&&(restant > 0)){
    if(restant >= 8){
      ino_map[byte_cpt]=255;
      restant-=8;
    }else{
      switch (restant){
      case 1: ino_map[byte_cpt]=128;restant=0;break;
      case 2: ino_map[byte_cpt]=192;restant=0;break;
      case 3: ino_map[byte_cpt]=224;restant=0;break;
      case 4: ino_map[byte_cpt]=240;restant=0;break;
      case 5: ino_map[byte_cpt]=248;restant=0;break;
      case 6: ino_map[byte_cpt]=252;restant=0;break;
      case 7: ino_map[byte_cpt]=254;restant=0;break;
      }
    }
    byte_cpt++;
  }
  while(byte_cpt <128){
 
    ino_map[byte_cpt]=0;//le reste est initialis  0
    byte_cpt++;
  }
  return restant;
}

int init_ino_map(USHORT disc_num,USHORT nb_inode){
  bloc bloc_ino;
  int restant;
  USHORT bloc_num=2;

  restant=nb_inode;
  while(restant > 0){
    restant=calc_ino_map(bloc_ino,restant);
    if(write_map_bloc(disc_num,bloc_ino,bloc_num)==-1) return -1;
    bloc_num++;
  }

  return 1;
}

int ino_sync(){
  int i;
  ino_addr *adresse;
  disc *disque;
  bloc_inode ino_bloc;

  for(i=0;i<MAX_NODE;i++){
    if(inode_table[i].open_cpt != 0){
      disque=&disc_table[i];
      /*recuperation de l'adresse*/
      to_ino_addr(adresse,disque,inode_table[i].inode_num);
      /*recuperation du bloc contenant l'inode*/
      if(read_inode(disque->dev_num,&ino_bloc,adresse->bloc_num)==-1) return -1;
      /*changement des donne*/
      ino_bloc[adresse->indice]=inode_table[i].node;
      /*ecriture du bloc*/
      if(write_inode(disque->dev_num,&ino_bloc,adresse->bloc_num)==-1) return -1;
    }
  }
  return 1;
}


/*convertion d'un offset en addresse, attention, il faut que le bloc d'arriv est dja t attribu sous 
*peine de se retrouver n'importe ou
*/
int offset2addr(USHORT disc_num,USHORT node_num,int offset,byte_addr *adresse){
  int ind_log_blocs, nb_bloc_index,i;
  ino_addr ino_adresse;
  disc *disque;
  inode ino;
  bloc_inode bloc_ino;
  USHORT next;
  data_index d_index;

  disque=&disc_table[disc_num];

  /*convertion de l'offset en indice de blocs logiques*/
  ind_log_blocs=offset/128;
  /*convertion en nombre de blocs d'index  parcourir*/
  if(ind_log_blocs <10){
    nb_bloc_index=0;
  }else{
    ind_log_blocs-=10;
    nb_bloc_index=ind_log_blocs/63 + 1;
    ind_log_blocs=ind_log_blocs%63;
  }


  /*lecture dans la partie de l'inode*/

  if(read_ino(disc_num,&ino,node_num)==-1) return -1;

  /*construction de l'addresse selon les cas*/
  if(nb_bloc_index==0){
    adresse->bloc_num=ino.blocs[ind_log_blocs];
    adresse->index=-1;
    adresse->indice=ind_log_blocs;
  }else{
 


     next=ino.blocs[10];
  
      for(i=nb_bloc_index-1;i>0;i--){
	  if(read_data_index(disc_num,&d_index,next)==-1)return -1;
	  next=d_index.next;      
      }
      if(read_data_index(disc_num,&d_index,next)==-1)return -1;
      adresse->bloc_num=d_index.blocs[ind_log_blocs];
      adresse->index=next;
      adresse->indice=ind_log_blocs;
    
  }
  return 1;
}


/*lecriture d'un inode sur le disque*/
int write_ino(USHORT disc_num,inode ino,USHORT ino_num){
  disc * disque;
  ino_addr ino_adresse;
  bloc_inode bloc_ino;
  disque=&disc_table[disc_num];
  int indice,i;
  /*verifie si le noeud n'est pas en mememorie et si c'est le cas, il l'ecrit dans la table es noeuds*/
  if((indice=node_in_mem (disc_num,ino_num))!=-1){
    //inode_table[indice].node=ino;
    inode_table[indice].node.etat=ino.etat;
    inode_table[indice].node.type=ino.type;
    inode_table[indice].node.taille=ino.taille;
    inode_table[indice].node.liens=ino.liens;
    inode_table[indice].node.icone=ino.icone;
    for(i=0;i<11;i++){
      inode_table[indice].node.blocs[i]=ino.blocs[i];
    }
  }else{
    to_ino_addr(&ino_adresse,disque,ino_num);
    //printf("write ino : disc num=%d bloc_num=%d indice=%d\n",ino_adresse.disc_num,ino_adresse.bloc_num,ino_adresse.indice);
    //printf("le type est %d\n",ino.type);
    if(read_inode(disc_num,&bloc_ino,ino_adresse.bloc_num)==-1) return -1;
    bloc_ino[ino_adresse.indice]=ino;
    //printf("bloc ecrit par write ino\n");
    if(write_inode(disc_num,&bloc_ino,ino_adresse.bloc_num)==-1) return -1;
  }
  return 1;
}



/*lecture d'un inode sur le disque*/
int read_ino(USHORT disc_num,inode *ino,USHORT ino_num){
  disc * disque;
  ino_addr ino_adresse;
  bloc_inode bloc_ino;
  int i,indice;
  /*verifie si le noeud n'est pas en memoire et si c'est le cas , le charge  partir de la table des noeuds*/
  if((indice=node_in_mem(disc_num,ino_num))!=-1){
    //printf("l'inode est en memoire\n");
    //memcpy(ino,&(inode_table[indice].node),sizeof(ino));
    ino->etat=inode_table[indice].node.etat;
    ino->type=inode_table[indice].node.type;
    ino->taille=inode_table[indice].node.taille;
    ino->liens=inode_table[indice].node.liens;
    ino->icone=inode_table[indice].node.icone;
    for(i=0;i<11;i++){
      ino->blocs[i]=inode_table[indice].node.blocs[i];
    }
  }else{
    disque=&disc_table[disc_num];
    to_ino_addr(&ino_adresse,disque,ino_num);
    //printf("bloc lu dans read_ino\n");
    if(read_inode(disc_num,&bloc_ino,ino_adresse.bloc_num)==-1) return -1;
    //printf("fin de la lecture dans read_ino\n");
    //memcpy(ino,&bloc_ino[ino_adresse.indice],sizeof(*ino));
    ino->etat=bloc_ino[ino_adresse.indice].etat;
    ino->type=bloc_ino[ino_adresse.indice].type;
    ino->taille=bloc_ino[ino_adresse.indice].taille;
    ino->liens=bloc_ino[ino_adresse.indice].liens;
    ino->icone=bloc_ino[ino_adresse.indice].icone;
    for(i=0;i<11;i++){
      ino->blocs[i]=bloc_ino[ino_adresse.indice].blocs[i];
    }
    //printf("fin de larecopie dans l'adrsesse donne\n");
  }
  return 1;
}

/*convertion d'une reference absolue ou relative en un numero d'inode*/
int ref2ino(USHORT disc_num,char *ref){
  char *curseur;//pour parcourir la reference
  char *cur_dir_name,name[40],dir_entr_no;
  int cur_node;//adresse de l'inode courant
  int cpt;
  entree_repertoire dir_entr;
  disc *disque;

  //printf("entre dans ref2ino\n");
  disque=&disc_table[disc_num];

  /*test sur le premier caractre pour determiner si la reference est absolue ou relative*/
  if(ref[0]=='/'){//reference absolue
    cur_node=0;
    curseur=ref+1;
    while(*curseur == '/') ref ++;//evite d'etre embete par les reference du type ///
    cur_dir_name=curseur;
  }else{
    cur_node=disque->cur_dir;
    curseur=ref;
    while(*curseur == '/') ref ++;
    cur_dir_name=ref;
  }
  //printf("test sur les reference relatives et absolues effectue\n");
  while(*curseur != '\0'){
    cpt=0;
    while((*curseur != '\0') && (*curseur != '/')){
      curseur++;
      cpt++;
    }
    if(cpt >40){
      ifs_err=NAME_TOO_LONG;
      return -1;
    }
    strncpy(name,cur_dir_name,cpt);
    name[cpt]='\0';
    //printf("le nom est %s\n",name);
    if((dir_entr_no=dir_entry_num(disc_num,cur_node,name))==-1) return -1;
    if(read_dir_entr(disc_num,&dir_entr,cur_node,dir_entr_no)==-1) return -1;
    cur_node=dir_entr.numero;
    while (*curseur=='/') curseur++;
    cur_dir_name=curseur;
  }
  return cur_node;
}



/*convertion d'une reference absolue ou relative dont l' un des membre n'a pas ete cree en un numero d'inode du pere et le champs
*new ref contient le nom qui n'existe pas et renvoie une erreur si l'entree existe 
*/
int newref2ino(USHORT disc_num,char *ref,char *newref){
  char *curseur;//pour parcourir la reference
  char *cur_dir_name,name[40],dir_entr_no;
  int cur_node;//adresse de l'inode courant
  int cpt;
  entree_repertoire dir_entr;
  disc *disque;

  disque=&disc_table[disc_num];

  /*test sur le premier caractre pour determiner si la reference est absolue ou relative*/
  if(ref[0]=='/'){//reference absolue
    cur_node=0;
    curseur=ref+1;
    while(*curseur == '/') ref ++;//evite d'etre embete par les reference du type ///
    cur_dir_name=curseur;
  }else{
    cur_node=disque->cur_dir;
    curseur=ref;
    while(*curseur == '/') ref ++;
    cur_dir_name=ref;
  }

  while(*curseur != '\0'){
    cpt=0;
    while((*curseur != '\0') && (*curseur != '/')){
      curseur++;
      cpt++;
    }
    if(cpt >40){
      ifs_err=NAME_TOO_LONG;
      return -1;
    }
    strncpy(name,cur_dir_name,cpt);
    name[cpt]='\0';
    if((dir_entr_no=dir_entry_num(disc_num,cur_node,name))==-1){
      if(ifs_err != FILE_DO_NOT_EXISTS){
	return -1;
      }else{//ratrapage de l'erreur
	strcpy(newref,name);
	return cur_node;
      }
    }
    if(read_dir_entr(disc_num,&dir_entr,cur_node,dir_entr_no)==-1) return -1;
    cur_node=dir_entr.numero;
    while (*curseur=='/') curseur++;
    cur_dir_name=curseur;
  }
  if(*curseur == '\0'){
    ifs_err=DIR_ENTRY_ALREADY_EXISTS;
    return -1;
  }
  return cur_node;
}

/**********************************fonctions pour le serveur***********************************/
void status(requete *query, reponse *rep){
  char  dir_name[512], disc_name[30];
  int disc_num,node_num,cpt;
  inode ino;
  //printf("entre dans status\n");

  /*recuperation du nom du disc et du repertoire*/
  if(parse_discref(query->args,disc_name,dir_name)==-1){
    rep->err=-1;
    return;
  }
  //printf("lenom du disc est %s et le nom du repertoire est %s\n",disc_name,dir_name);
  /*recuperation du numero du disc*/
  if(*disc_name=='\0'){
    disc_num=cur_disc;
  }else{
    /*on cher che le numero*/
    disc_num=hash_func(disc_name);
    while ((cpt<MAX_DEV)&&
	   ((is_empty(disc_table[disc_num]))||
	    (strcmp((char *)disc_table[disc_num].name,disc_name)))){
      disc_num++;
      cpt++;
    }
    if(cpt>=MAX_DEV){
      ifs_err=NO_SUCH_DEV;
      rep->err==-1;
      return;
    }
  }
  /*recuperation du numero du noeud*/
  if((node_num=ref2ino(disc_num,dir_name))==-1){
    rep->err=-1;
    return;
  }
  //printf("le numero du disque est %d le numero du repertoire est %d\n",disc_num,node_num);
  /*recuperation de la structure d'inode*/
  if(read_ino(disc_num,&ino,node_num)==-1){
    rep->err=-1;
    return;
  }
  /*remplissage de la structure reponse*/
  rep->msg_num[0]=(USHORT)disc_num;
  rep->msg_num[1]=(USHORT)node_num;
  rep->msg_num[2]=ino.type;
  rep->msg_num[3]=ino.liens;
  rep->msg_num[4]=ino.icone;
  rep->msg_num[5]=ino.taille;

  rep->err=0;
}

/*recuperationn de l'inode par le descripteur*/
void fstatus(requete *query, reponse *rep){
  inode ino;
  //printf("entre dans fstatus\n");



  /*recuperation de la structure d'inode*/
 ino=inode_table[o_file_table[query->args_int].mem_node_num].node;
 /*remplissage de la structure reponse*/
 rep->msg_num[0]=(USHORT)inode_table[o_file_table[query->args_int].mem_node_num].disc_num;
 rep->msg_num[1]=(USHORT)inode_table[o_file_table[query->args_int].mem_node_num].inode_num;
 rep->msg_num[2]=ino.type;
 rep->msg_num[3]=ino.liens;
 rep->msg_num[4]=ino.icone;
 rep->msg_num[5]=ino.taille;
  
 rep->err=0;
}


/*liaison d'un fichier avec un fichier icone*/
void lnico(requete *query, reponse *rep){
  char file_ref[512],icone_ref[512],disc_ico_name[32],disc_file_name[32],file_dir_ref[512],icone_dir_ref[512];
  char *cur_ico_name;
  int cpt,ico_num,file_num,disc_file_num,disc_ico_num,mem_ind;
  inode ino;
  /*recuperation des references du fichier et de l'icone*/
  //printf("la chaine al 'entree est %s\n",query->args);
  cur_ico_name=query->args;
  cpt=0;
  while((*cur_ico_name != ' ')&&(*cur_ico_name!='\0')){
    cur_ico_name++;
    cpt++;
  }
  if(*cur_ico_name !='\0') cur_ico_name++;
  strncpy(file_ref,query->args,cpt);
  file_ref[cpt]='\0';
  strcpy(icone_ref,cur_ico_name);

  /*recupeartion du nom u disc et du chemin pour le fichier*/
  parse_discref(file_ref,disc_file_name,file_dir_ref);
  //printf("lnico : fichier_name = %s\n",file_dir_ref);
  /*recuperation du nom du disc et du chemein pour l'icone*/
  parse_discref(icone_ref,disc_ico_name,icone_dir_ref);
  //printf("lnico : icone_name = %s\n",icone_dir_ref);
  /*recuperation du numero du disc du fichier et de son numero d'inode*/
  if(*disc_file_name == '\0'){
    disc_file_num=cur_disc;
  }else{
    if((disc_file_num=discname2num(disc_file_name))==-1) {rep->err=-1;return;}
  }
  if((file_num=ref2ino(disc_file_num,file_dir_ref))==-1) {rep->err=-1;return;}

  /*recuperation du numero du disc de l'icone et de son numero d'inode*/
 if(*disc_ico_name == '\0'){
    disc_ico_num=cur_disc;
  }else{
    if((disc_ico_num=discname2num(disc_ico_name))==-1) {rep->err=-1;return;}
  }
  if((ico_num=ref2ino(disc_ico_num,icone_dir_ref))==-1) {rep->err=-1;return;}

  if(disc_ico_num != disc_file_num){
    ifs_err=ICO_DIFF_DISC;
    {rep->err=-1;return;}
  }
  /*verification que le fichier icone est ien un fichier icone et que le fichier de destination n'est pas un fichier icone*/
  if(read_ino(disc_file_num,&ino,ico_num)==-1){rep->err=-1;return;}
  if(!IS_ICO(&ino)){
    ifs_err=NO_ICO_LINK;
    rep->err=-1;
    return;
  }
  if(read_ino(disc_file_num,&ino,file_num)==-1){rep->err=-1;return;}
  if(IS_ICO(&ino)){
    ifs_err=ICO_LINK_ICO;
    rep->err=-1;
    return;
  }
  ino.icone=ico_num;  /*changement du champs icone dans l'inode du fichier*/
  /*si le oeud est en memoire on change dans la memeoire sinon on ecrit le noed dans la table des caches*/
  if((mem_ind=node_in_mem(disc_file_num,file_num))==-1){
    if(write_ino(disc_file_num,ino,file_num)==-1){rep->err=-1;return;}
  }else{
    inode_table[mem_ind].node=ino;
  }
  rep->err=0;
}

/*suppresion d'un lien vers une icone*/
void ulnico(requete *query,reponse *rep){
  char disc_name[32],file_name[512];
 int disc_num,file_num,mem_ind;
 inode ino;
 /*recuperation du nom du disque et de a reference du fichier*/
 parse_discref(query->args,disc_name,file_name);
 /*recuperation du numero du dique et du numero d'inode du fichier*/
 if(*disc_name=='\0'){
   disc_num=cur_disc;
 }else{
    if((disc_num=discname2num(disc_name))==-1){rep->err=-1;return;}
 }
 if((file_num=ref2ino(disc_num,file_name))==-1){rep->err=-1;return;}
 /*changement de la valeur du champs icone dans l'inode*/
 if(read_ino(disc_num,&ino,file_num)==-1){rep->err=-1;return;}
 ino.icone=0;
 /*si le oeud est en memoire on change dans la memeoire sinon on ecrit le noed dans la table des caches*/
 if((mem_ind=node_in_mem(disc_num,file_num))==-1){
   if(write_ino(disc_num,ino,file_num)==-1){rep->err=-1;return;}
 }else{
   inode_table[mem_ind].node=ino;
 }
 rep->err=0;
}
