/****************************************************************
*Fichier cree par :                                             *
*                 Selma Abdelrahim                              *
*                 Farzana Chutkai                               *
*                 Denis Berthod                                 *
*Pour le projet du cours de systeme du premier semestre         *
*de l'anne 2003/2004 : Le systeme IFS                          *
*****************************************************************/

#ifndef IFS_TYPES
#define IFS_TYPES

#include "lendian.h"
#include "blocio.h"

//nombre maximum de disque
#define MAX_DEV 10

//nombre maximum de noeud en memoire
#define MAX_NODE 32

//nombre maximum de fichier ouvert 
#define MAX_OPEN 64

//nombre maximum de caches
#define MAX_CACHE 50

//nombre maximum de sous-repertoire dans une reference
#define MAX_DIR_PROF 20

//magic number
#define MAGIC (UINT)0x49465300

//adresse du pointeur null
#define IFS_NULL (USHORT)1

//type des descripteur de fichier
typedef int desc_t;

//type des position des fichiers
typedef int pos_t;
  

//type des disques
typedef struct disc{
  int empty;//est diffrent de 0 pour reprsent une structure disc vide
  USHORT cur_dir;//inode du repertoire courant, cette varaible n'est pas garde en memeoire sur le disque
  USHORT dev_num;//numero du disque
  char *name[32];//nom du volume
  USHORT description_size;//nombre de blocs de la zone de description
  USHORT node_size;//nombre de blocs pour la zone des inodes
  USHORT data_size;//nombre de blocs pour la zone des donnes
  USHORT last_addr;//addresse du dernier bloc
  USHORT free_data_addr;//adresse d'un bloc libre de la memoire
  USHORT free_data_space;//nombre de blocs de donnes libres
  USHORT inode_addr;//adresse de la table des inodes
  USHORT free_inode_space;//nombres d'inodes libres
  USHORT quick_inode[32];//tableau d'inode en acces rapide
  USHORT quick_inode_cpt;//nombre d'inodes restant en acces rapide
  USHORT quick_inode_ind;//indice du prochain inode libre dans le tableau quick_inode 
}disc;

//fonction d'ecriture et de lecture d'un bloc disque
int write_disc(int descripteur,disc *disque);

int read_disc(int descripteur,disc *disque);

//type des i_noeuds
typedef struct inode{
  USHORT etat;
  USHORT type;
  USHORT taille;//pour un repertoire , reprsente le nombre d'entrees sinon le nombre d'octet du fichier
  USHORT liens;
  USHORT icone;
  USHORT blocs[11];
}inode;

#define NOEUD_LIBRE (USHORT)0
#define NOEUD_UTILISE (USHORT)1
#define EST_LIBRE(pn) ((pn)->etat==NOEUD_LIBRE)
#define ORDINAIRE (USHORT)0 
#define REPERTOIRE (USHORT)1
#define ICONE (USHORT)2
#define IS_ORD(pn) ((pn)->type==ORDINAIRE)
#define IS_REP(pn) ((pn)->type==REPERTOIRE)
#define IS_ICO(pn) ((pn)->type==ICONE)

//structure des blocs d'inode
typedef inode bloc_inode[4];

//fonction de lecture et d'ecriture d'un bloc_inode sur disque  l'emplacement no_bloc 
int write_inode(USHORT disc_num,bloc_inode *b_ino,USHORT no_bloc); 

int read_inode(USHORT disc_num,bloc_inode *b_ino,USHORT no_bloc);

/*structure stat permettant de recuprer des inofs sur un node*/
typedef struct ifsstat{
  USHORT disc_num;//numero du disque contenant le fichier
  USHORT node_num;//numero du noeud sur ce disc
  USHORT type;//type du fichier
  USHORT liens;//nb de liens vers se noeud
  USHORT icone;//numero du noeud de l'icone associ a ce fichier
  USHORT taille;//taille du fichier en octet pour les fichier ordinaire et es icones ou le nombre d'entrees pour un repertoire
}ifsstat;

/*type de pointeur sur ifs_stat*/
typedef ifsstat *stat_p;

//type des blocs-index de donnes
typedef struct data_index{
  USHORT blocs[63];//tableau de pointeurs vers des blocs de donnes
  USHORT next;//pointeur vers le bloc-index suivant
}data_index;

//fonction de lecture et d'ecriture d'un data_index sur le disque  l'emplacement no_bloc
int write_data_index(USHORT disc_num,data_index *d_index,USHORT no_bloc);

int read_data_index(USHORT disc_num,data_index *d_index,USHORT no_bloc);

//type des entrs de repertoires
typedef struct entree_repertoire{
  unsigned char nom[30];//nom du lien associ au noeud
  USHORT numero;//numero du noeud associ au nom
}entree_repertoire;

//type des pointeur sur les entrees de repertoire
typedef entree_repertoire *entreerep_p;

//type des blocs repertoires
typedef entree_repertoire bloc_rep[4];


//fonction de lecture et d'ecriture d'un bloc de repertoire
int write_bloc_rep(USHORT disc_num,bloc_rep *b_rep,USHORT no_bloc);

int read_bloc_rep(USHORT disc_num,bloc_rep *b_rep,USHORT no_bloc);

//type des blocs de donnes allouables
typedef struct free_bloc{
  USHORT next_index;//addresse du prochain bloc d'index
  USHORT cpt;//nombre de blocs dans le tableau
  USHORT blocs[62];//adresses de 62 blocs allouables
}free_bloc;

#define EST_ALLOUABLE(pf) ((pf)->cpt==0)

//fonctions d'criture et de lecture des blocs libres
int write_free_bloc(USHORT disc_num,free_bloc *free_b,USHORT no_bloc);

int read_free_bloc(USHORT disc_num,free_bloc *free_b,USHORT no_bloc);

//fonction de'criture et de lecture d'un bloc de donnes quelconque (pas de structure particulire)
int write_bloc(USHORT disc_num,bloc b,USHORT no_bloc);

int read_bloc(USHORT disc_num,bloc b,USHORT no_bloc);

//fonction de lecture et d'ecriture d'un bloc inode_map(pas de structures particulire)
int write_map_bloc(USHORT disc_num,bloc map_b,USHORT no_bloc);

int read_map_bloc(USHORT disc_num,bloc map_b,USHORT no_bloc);
#endif
